
import astropy.io.fits as pyfits
import numpy
import sys
import biggles
from scipy import signal

infile = sys.argv[1]
extension = 0
frame = pyfits.open(infile)

poly_order_x=10
poly_order_y=8
nsteps=100
max_gradient=5
orderdef=None


medianfiltersize = 5
minfiltersize  = 10

if nsteps <= poly_order_y: poly_order_y = int(nsteps / 2)

im = frame[extension].data
(ysize, xsize) = im.shape

bgim	= numpy.zeros((ysize, xsize),  dtype=numpy.float32)
xfitarr = numpy.zeros((nsteps, xsize), dtype=numpy.float32)

xx = numpy.arange(xsize, dtype=numpy.float32)
yy = numpy.arange(ysize, dtype=numpy.float32)

xx_mean = xx.mean()
yy_mean = yy.mean()

ystep = int(ysize / (nsteps - 1))

yvals = (numpy.arange(nsteps) * ystep).round()

ycount = 0


for yind in yvals:
  #medianvec = numpy.zeros(xsize, dtype=numpy.float32)
  minvec  = numpy.zeros(xsize, dtype=numpy.float32)

  ymin_ind = numpy.max([yind-medianfiltersize, 0])
  ymax_ind = numpy.min([yind+medianfiltersize, ysize-1])

  
  naver = ymax_ind - ymin_ind + 1
  
  meanvec = numpy.average(im[ymin_ind:ymax_ind, :], axis=0)
  kernel = numpy.repeat(0.2, 5)
  d_meanvec = numpy.gradient(numpy.convolve(meanvec, kernel, mode='sample'))
  d_meanvec[0:10] = 0
  d_meanvec[xsize-11:xsize-1] = 0

  order_throughs = []
  order_throughs2 = []
  order_peaks = []
  all_peaks = []



  # First try to detect the peaks myself...
  for j in numpy.arange(xsize-1) + 1:
   if (d_meanvec[j-1] > max_gradient and d_meanvec[j] < max_gradient): all_peaks.append(j)


  # Now make use of existing orderdef if it exists
  if orderdef is not None:

    for order in orderdef:
      centery  = order['centerpix']
      centerx  = order['centerval']

      cheb_min = order['coefs'][2]
      cheb_max = order['coefs'][3]
      chebcoef = order['coefs'][4:]

      orderpos = _chebev(cheb_min, cheb_max, chebcoef, numpy.array([yind+1]))
      order_peaks.append(int(orderpos[0] + centerx) - 1)

    # But also append 'own' peaks below and above defined orders
    min_order_peak = numpy.min(order_peaks)
    max_order_peak = numpy.max(order_peaks)
    for peak in all_peaks:
      if peak < min_order_peak: order_peaks.append(peak)
      if peak > max_order_peak: order_peaks.append(peak)

  else:
    # Use whatever we have
    order_peaks = all_peaks
 
  order_peaks = numpy.array(order_peaks)   
  npeaks = order_peaks.size
  order_throughs = (order_peaks[1:] + order_peaks[:npeaks-1]) / 2

  # Correct the max gradient for frames with very low pixel counts
  max_gradient = numpy.min((5, numpy.mean(meanvec.take(order_peaks)) / 4))


  # Now start searching for points that sample the background
  # (scattered light level)

  # First, find low points before first order, with
  # at least one point every 20 pixels
  for xpix in numpy.arange(0, order_peaks.min(), 20):
    shortvec = numpy.arange(20) + xpix
    shortvec = numpy.extract(shortvec < order_peaks.min(), shortvec)

    if (shortvec.size > 0):
      order_throughs2.append(shortvec[numpy.argmin(meanvec.take(shortvec))])

  # Then loop over orders and find the troughs between the orders
  for xpix in order_throughs:
    shortvec = numpy.arange(11) - 5 + xpix
    shortvec = numpy.extract(shortvec > 1,	 shortvec)
    shortvec = numpy.extract(shortvec < xsize-1, shortvec)
    short_dd = d_meanvec.take(shortvec)

    # Use gradient to find region between peaks
    (inds,) = numpy.where(short_dd < -max_gradient)
    if inds.size == 0: inds = 0
    min_ind = numpy.max(inds)
    (inds,) = numpy.where(short_dd >  max_gradient)
    if inds.size == 0: inds = short_dd.size
    max_ind = numpy.min(inds)

    # Add the found region to the array of troughs
    nfound = 0
    for j in numpy.arange(min_ind, max_ind - min_ind):
      order_throughs2.append(shortvec[j])
      nfound = nfound + 1
    if nfound == 0:
      # But, if no through found, try to take minimum value of meanvec at position of trough
      try: order_throughs2.append(shortvec[numpy.argmin(meanvec.take(shortvec))])
      except: pass

      # Old solution - middle point between the order peaks
      # Does not work well because of non-monotonous order spacing
      # order_throughs2.append(xpix)

    # Safety catch: remove any incorrectly selected order peaks...
    try: order_throughs2.remove(shortvec[j])
    except: pass



  # Finally, find lowest point after last order
  shortvec = numpy.arange(50) + order_peaks.max()
  shortvec = numpy.extract(shortvec < xsize-1, shortvec)
  short_dd = d_meanvec.take(shortvec)

  if shortvec.size > 0:
    order_throughs2.append(shortvec[numpy.argmin(meanvec.take(shortvec))])


  # Finally, make sure each point is only selected once
  order_throughs = numpy.unique(order_throughs2)


  niter = 0
  # Perform fitting with sigma-clipping
  while 1:

    coefs = numpy.polyfit(order_throughs - xx_mean, meanvec.take(order_throughs), poly_order_x)
    xfit_poly = numpy.poly1d(coefs)
    xfit = xfit_poly(order_throughs - xx_mean)

    sigma = numpy.abs(meanvec.take(order_throughs) - xfit) / numpy.std(meanvec.take(order_throughs) - xfit)
    rejected = numpy.extract(sigma > 3, order_throughs)
    order_throughs = numpy.extract(sigma < 3, order_throughs)
    
    niter = niter + 1
    if niter == 5 or rejected.size == 0: break
  
  xfit = xfit_poly(xx - xx_mean)
  xfitarr[ycount, :] = xfit

  minloc = signal.argrelextrema(meanvec, numpy.less)[0]
  minlocflux = meanvec[minloc]


  #Debugging plot in y-direction (along the orders)
  #biggles.configure('screen', 'width',  900)
  #biggles.configure('screen', 'height', 450)
  #fr = biggles.FramedPlot()
  #fr.yrange = [bgim[:, xind].min()-50, bgim[:, xind].max()+50]
  #fr.add(biggles.Curve(minloc, minlocflux, color='blue'))
  #fr.add(biggles.Points(yvals, xfitarr[:, xind], color='red'))
  #fr.add(biggles.Points(yvals.take(goodind), xfitarr[goodind, xind], color='green'))
  #fr.title = str(xind)
  #fr.show()

  niter = 0
  # Perform fitting with sigma-clipping
  while 1:

    coefs2 = numpy.polyfit(minloc, minlocflux, poly_order_x)
    xfit_poly2 = numpy.poly1d(coefs)
    xfit2 = xfit_poly2(minloc)

    sigma2 = numpy.abs(minlocflux - xfit2) / numpy.std(minlocflux - xfit2)
    rejected2 = numpy.extract(sigma2 > 3, minloc)
    minloc = numpy.extract(sigma2 < 3, minloc)
    minlocflux = meanvec[minloc]
    
    niter = niter + 1
    if niter == 5 or rejected2.size == 0: break
  
  xfit2 =  xfit_poly2(xx - xx_mean)
  xfitarr[ycount, :] = xfit2


  ycount = ycount + 1


  # Debugging plot in x-direction (across the orders)
  #biggles.configure('screen', 'width',  900)
  #biggles.configure('screen', 'height', 900)
  #fr = biggles.FramedPlot()
  #fr.yrange = [5000, meanvec.take(order_throughs).max()+10000]
  #fr.add(biggles.Curve(xx, xfit2, color='green'))
  #fr.add(biggles.Curve(xx, xfit, color='blue'))
  #fr.add(biggles.Points(order_throughs, meanvec.take(order_throughs)))
  #fr.add(biggles.Points(order_peaks, meanvec.take(order_peaks), color='red'))
  #fr.title = str(yind)
  #fr.show()


# Now fit in the y-direction
for xind in numpy.arange(xsize):

  # Perform fitting with sigma-clipping
  niter = 0
  goodind = numpy.arange(nsteps)

  while 1:

    coefs = numpy.polyfit(yvals.take(goodind) - yy_mean, xfitarr[goodind, xind], poly_order_y)

    yfit_poly = numpy.poly1d(coefs)
    yfit = yfit_poly(yvals.take(goodind) - yy_mean)

    sigma = (xfitarr[goodind, xind] - yfit) / numpy.std(xfitarr[goodind, xind] - yfit)
    rejected = numpy.extract(sigma > 3, goodind)
    goodind  = numpy.extract(sigma < 3, goodind)

    niter = niter + 1
    if niter == 3 or rejected.size == 0 or goodind.size == 0: break

  if goodind.size == 0: 
    print "Error: no points left when y-fitting the background"
    coefs=numpy.polyfit(xfitarr[:, xind])
  bgim[:, xind] = yfit_poly(yy - yy_mean)

outfile = "mytest.fits"
pyfits.writeto(outfile, bgim, clobber=True)

  #Debugging plot in y-direction (along the orders)
  #biggles.configure('screen', 'width',  900)
  #biggles.configure('screen', 'height', 450)
  #fr = biggles.FramedPlot()
  #fr.yrange = [bgim[:, xind].min()-50, bgim[:, xind].max()+50]
  #fr.add(biggles.Curve(yy, bgim[:, xind], color='blue'))
  #fr.add(biggles.Points(yvals, xfitarr[:, xind], color='red'))
  #fr.add(biggles.Points(yvals.take(goodind), xfitarr[goodind, xind], color='green'))
  #fr.title = str(xind)
  #fr.show()
frame[extension].data = frame[extension].data - bgim
outfile = "corrected.fits"
pyfits.writeto(outfile, frame[extension].data, clobber=True)
